--------------------------------------------------------------------------------
-- ReSynth Tool
--
-- Copyright 2011 Martin Bealby
--
-- Action code - Wrapper glue between the midimaps / gui and the oscillators
--
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------
function set_oscillator_type(osc, type_id)
  if type_id == ANALOG_OSC then
    -- wavetable -> analog
    osc_wt[osc]:unbind()
    osc_an[osc]:bind(inst.samples[osc])
    osc_type[osc] = type_id
    osc_an[osc]:render()
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    -- analog -> wavetable
    osc_an[osc]:unbind()
    osc_wt[osc]:bind(inst.samples[osc])
    osc_type[osc] = type_id
  end
  show_waveform(osc)
end


function set_waveform(osc, waveform)
  osc_an[osc]:set_waveform(waveform)
  inst.samples[osc].name = osc_an[osc]:get_settings()
  show_waveform(osc)
end


function set_wavemod(osc, wavemod)
  osc_an[osc]:set_wavemod(wavemod)
  inst.samples[osc].name = osc_an[osc]:get_settings()
  show_waveform(osc)
end


function set_wavemod_lfo_depth(osc, depth)
  osc_an[osc]:set_wavemod_lfo_depth(depth)
  inst.samples[osc].name = osc_an[osc]:get_settings()
  show_waveform(osc)
end


function set_wavemod_lfo_rate(osc, rate)
  osc_an[osc]:set_wavemod_lfo_rate(rate)
  inst.samples[osc].name = osc_an[osc]:get_settings()
  show_waveform(osc)
end


function set_smoothing(osc, toggle)
  osc_an[osc]:set_smooth(toggle)
  osc_wt[osc]:set_smooth(toggle)
  if osc_type[osc] == ANALOG_OSC then
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    inst.samples[osc].name = osc_wt[osc]:get_settings()
  end
  show_waveform(osc)
end


function set_transpose(osc, transpose)
  osc_an[osc]:set_transpose(transpose)
  osc_wt[osc]:set_transpose(transpose)
  if osc_type[osc] == ANALOG_OSC then
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    inst.samples[osc].name = osc_wt[osc]:get_settings()
  end
  show_waveform(osc)
end


function set_fine_tune(osc, fine_tune)
  osc_an[osc]:set_fine_tune(fine_tune)
  osc_wt[osc]:set_fine_tune(fine_tune)
  if osc_type[osc] == ANALOG_OSC then
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    inst.samples[osc].name = osc_wt[osc]:get_settings()
  end
  show_waveform(osc)
end


function set_volume(osc, volume)
  osc_an[osc]:set_volume(volume)
  osc_wt[osc]:set_volume(volume)
  if osc_type[osc] == ANALOG_OSC then
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    inst.samples[osc].name = osc_wt[osc]:get_settings()
  end
  show_waveform(osc)
end


function set_panning(osc, panning)
  osc_an[osc]:set_panning(panning)
  osc_wt[osc]:set_panning(panning)
  if osc_type[osc] == ANALOG_OSC then
    inst.samples[osc].name = osc_an[osc]:get_settings()
  else
    inst.samples[osc].name = osc_wt[osc]:get_settings()
  end
  show_waveform(osc)
end


function show_waveform(osc)
  -- Maybe select an oscillator waveform
  if renoise.app().window.active_middle_frame ==
    renoise.ApplicationWindow.MIDDLE_FRAME_SAMPLE_EDITOR then
    renoise.song().selected_sample_index = osc
  end
end


function global_reset()
  -- Power off, power on :)
  for osc = 1, 4 do
    set_oscillator_type(osc, ANALOG_OSC)
    osc_controls[osc][CONTROL_OSC_TYPE].value = ANALOG_OSC
    
    set_waveform(osc, ANALOGOSC_WAVEFORM_SINE)
    osc_controls[osc][CONTROL_ANALOG_WAVEFORM].value = ANALOGOSC_WAVEFORM_SINE
    
    set_wavemod(osc, 0)
    osc_controls[osc][CONTROL_ANALOG_WAVEMOD].value = 0
    update_controls(osc, CONTROL_ANALOG_WAVEMOD)
    
    set_wavemod_lfo_depth(osc, 0)
    osc_controls[osc][CONTROL_ANALOG_LFO_DEPTH].value = 0
    update_controls(osc, CONTROL_ANALOG_LFO_DEPTH)
    
    set_wavemod_lfo_rate(osc, 0)
    osc_controls[osc][CONTROL_ANALOG_LFO_RATE].value = 0
    update_controls(osc, CONTROL_ANALOG_LFO_RATE)
    
    set_smoothing(osc, true)
    osc_controls[osc][CONTROL_SMOOTHING].value = 1
    
    set_transpose(osc, 0)
    osc_controls[osc][CONTROL_TRANSPOSE].value = 0
    update_controls(osc, CONTROL_TRANSPOSE)
    
    set_fine_tune(osc, 0)
    osc_controls[osc][CONTROL_FINE_TUNE].value = 0
    update_controls(osc, CONTROL_FINE_TUNE)
    
    set_volume(osc, math.infdb)
    osc_controls[osc][CONTROL_VOLUME].value = -38
    update_controls(osc, CONTROL_VOLUME)
    
    set_panning(osc, 0)
    osc_controls[osc][CONTROL_PANNING].value = 0
    update_controls(osc, CONTROL_PANNING)
  end
  
  -- sound from osc 1 only (2.2 update)
  set_volume(1, -3)
  osc_controls[1][CONTROL_VOLUME].value = -3
  update_controls(1, CONTROL_VOLUME)

  update_status("ReSynth reset to defaults")
end


function randomize_osc()
  -- Randomise the oscillator settings
  local r = 0
  local rnd = math.random
  math.randomseed(os.time())
  
  for osc = 1, 4 do
    set_oscillator_type(osc, ANALOG_OSC)
    osc_controls[osc][CONTROL_OSC_TYPE].value = ANALOG_OSC
    
    r = rnd(#osc_controls[osc][CONTROL_ANALOG_WAVEFORM].items)
    set_waveform(osc, r)
    osc_controls[osc][CONTROL_ANALOG_WAVEFORM].value = r
    
    r = rnd(math.floor(osc_controls[osc][CONTROL_ANALOG_WAVEMOD].max))
    set_wavemod(osc, r)
    osc_controls[osc][CONTROL_ANALOG_WAVEMOD].value = r
    update_controls(osc, CONTROL_ANALOG_WAVEMOD)
    
    r = rnd(math.floor(osc_controls[osc][CONTROL_ANALOG_LFO_DEPTH].max))
    set_wavemod_lfo_depth(osc, r)
    osc_controls[osc][CONTROL_ANALOG_LFO_DEPTH].value = r
    update_controls(osc, CONTROL_ANALOG_LFO_DEPTH)
    
    r = rnd(math.floor(osc_controls[osc][CONTROL_ANALOG_LFO_RATE].max))
    set_wavemod_lfo_rate(osc, r)
    osc_controls[osc][CONTROL_ANALOG_LFO_RATE].value = r
    update_controls(osc, CONTROL_ANALOG_LFO_RATE)
    
    r = rnd(2)
    if r == 1 then
      set_smoothing(osc, true)
    else
      set_smoothing(osc, false)
    end
    osc_controls[osc][CONTROL_SMOOTHING].value = r
  end
  
  r = rnd(6)
  update_status("You throw a "..tostring(r).." and ReSynth randomizes your oscillator settings")
end


function randomize_tuningmixer()
  -- Randomise the oscillator tuning and mixer
  local r = 0
  local rnd = math.random
  math.randomseed(os.time())
  
  for osc = 1, 4 do
    r = rnd(49) - 25
    set_transpose(osc, r)
    osc_controls[osc][CONTROL_TRANSPOSE].value = r
    update_controls(osc, CONTROL_TRANSPOSE)
    
    r = rnd(127) - 64
    set_fine_tune(osc, r)
    osc_controls[osc][CONTROL_FINE_TUNE].value = r
    update_controls(osc, CONTROL_FINE_TUNE)
    
    r = rnd(40) - 38
    set_volume(osc, math.db2lin(r))
    osc_controls[osc][CONTROL_VOLUME].value = r
    update_controls(osc, CONTROL_VOLUME)
    
    r = rnd(101) - 51
    set_panning(osc, r)
    osc_controls[osc][CONTROL_PANNING].value = r
    update_controls(osc, CONTROL_PANNING)
  end
  
  update_status("You close your eyes and your tuning and mixer settings change")
end
