--------------------------------------------------------------------------------
-- ReSynth Tool
--
-- Copyright 2011 Martin Bealby
--
-- GUI code
--
-- This is a bit of a hack as each control affect the oscillator(s) as required
-- even if they are not bound - although they don't render. Probably best to
-- keep sane internal state
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Constants
--------------------------------------------------------------------------------
local STATUS_LINE_STARTUP = "ReSynth 2 initialised successfully :-)"
local CONTENT_SPACING = renoise.ViewBuilder.DEFAULT_CONTROL_SPACING
local DEFAULT_DIALOG_BUTTON_HEIGHT = renoise.ViewBuilder.DEFAULT_DIALOG_BUTTON_HEIGHT
local DEFAULT_DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
local MAIN_GROUP_STYLE = "border"
local CONTROL_GROUP_STYLE = "group" 
CONTROL_OSC_TYPE           = 1
CONTROL_ANALOG_WAVEFORM    = 2
CONTROL_ANALOG_WAVEMOD     = 3
CONTROL_ANALOG_LFO_DEPTH   = 4
CONTROL_ANALOG_LFO_RATE    = 5
CONTROL_WAVETABLE_WAVEBANK = 6
CONTROL_WAVETABLE_WAVEFORM = 7
CONTROL_SMOOTHING          = 8
CONTROL_TRANSPOSE          = 9
CONTROL_FINE_TUNE          = 10
CONTROL_VOLUME             = 11
CONTROL_PANNING            = 12


--------------------------------------------------------------------------------
-- Variables
--------------------------------------------------------------------------------
osc_controls = {{}, {}, {}, {}}
--[[
    [x] = oscillator
        [1]  = osc_type
        [2]  = analog.waveform
        [3]  = analog.wavemod
        [4]  = analog.lfo_amount
        [5]  = analog.lfo_rate
        [6]  = wavetable.wavebank
        [7]  = wavetable.waveform
        [8]  = common.smoothing
        [9]  = common.transpose
        [10] = common.fine_tune
        [11] = common.volume
        [12] = common.panning
]]--
status_line = nil

local resynth_view = nil
local resynth_dialog = nil
local vb = renoise.ViewBuilder()

                  
--------------------------------------------------------------------------------
-- Functions
--------------------------------------------------------------------------------
function update_status(message)
  -- Updates the status line in the ReSynth window
  assert(type(message) == "string")
  if status_line then
    status_line.text = message
  end
end


function update_controls(osc, controlid)
  -- Updates a GUI control (although NOT their values, confusing I know!)
  if osc_controls[osc][controlid] then
    if controlid == CONTROL_OSC_TYPE then
      if osc_type[osc] == ANALOG_OSC then
        update_status("Oscillator "..tostring(osc).." switched to analog mode")
      elseif osc_type[osc] == WAVETABLE_OSC then
        update_status("Oscillator "..tostring(osc).." switched to wavetable mode")
      end

    elseif controlid == CONTROL_ANALOG_WAVEFORM then
      local i = osc_an[osc]:get_waveform()
      if i == ANALOGOSC_WAVEFORM_SINE then
        update_status("Oscillator "..tostring(osc).." waveform set to sin/tri")
      elseif i == ANALOGOSC_WAVEFORM_PULSE then
        update_status("Oscillator "..tostring(osc).." waveform set to pulse")
      elseif i == ANALOGOSC_WAVEFORM_SAWTOOTH then
        update_status("Oscillator "..tostring(osc).." waveform set to sawtooth")
      elseif i == ANALOGOSC_WAVEFORM_NOISE then
        update_status("Oscillator "..tostring(osc).." waveform set to noise")
      end

    elseif controlid == CONTROL_ANALOG_WAVEMOD then
      local i = osc_an[osc]:get_wavemod()
      osc_controls[osc][CONTROL_ANALOG_WAVEMOD].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." wavemod set to "..tostring(i).."/63")

    elseif controlid == CONTROL_ANALOG_LFO_DEPTH then
      local i = osc_an[osc]:get_wavemod_lfo_depth()
      osc_controls[osc][CONTROL_ANALOG_LFO_DEPTH].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." wavemod LFO depth set to "..tostring(i).."/31") 

    elseif controlid == CONTROL_ANALOG_LFO_RATE then
      local i = osc_an[osc]:get_wavemod_lfo_rate()
      osc_controls[osc][CONTROL_ANALOG_LFO_RATE].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." wavemod LFO rate set to "..tostring(i).." wavecycles") 
    
    elseif controlid == CONTROL_WAVETABLE_WAVEBANK then
      local wbank = osc_wt[osc]:get_bank_list()[osc_controls[osc][CONTROL_WAVETABLE_WAVEBANK].value]
      update_status("Oscillator "..tostring(osc).." has selected a new wavebank: "..wbank) 

    elseif controlid == CONTROL_WAVETABLE_WAVEFORM then
      local wbank = osc_wt[osc]:get_bank_list()[osc_controls[osc][CONTROL_WAVETABLE_WAVEBANK].value]
      local wid = osc_wt[osc]:get_waveform_list(wbank)
      local index = osc_controls[osc][CONTROL_WAVETABLE_WAVEFORM].value
      update_status("Oscillator "..tostring(osc).." has loaded new wavetable waveform: "..wid[index]) 
   
    elseif controlid == CONTROL_SMOOTHING then
      local i = osc_an[osc]:get_smooth()
      if i then
        update_status("Oscillator "..tostring(osc).." smoothing is enabled")      
      else
        update_status("Oscillator "..tostring(osc).." smoothing is disabled")      
      end
    
    elseif controlid == CONTROL_TRANSPOSE then
      local i = osc_an[osc]:get_transpose()
      osc_controls[osc][CONTROL_TRANSPOSE].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." transpose set to "..tostring(i).." semitones")  
      
    elseif controlid == CONTROL_FINE_TUNE then
      local i = osc_an[osc]:get_fine_tune()
      osc_controls[osc][CONTROL_FINE_TUNE].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." fine tuning set to "..tostring(i).. " cents")
  
    elseif controlid == CONTROL_VOLUME then
      local i = osc_an[osc]:get_volume()
      if i == VOL_ABSMIN then
        osc_controls[osc][CONTROL_VOLUME].tooltip = "off"
        update_status("Oscillator "..tostring(osc).." volume set to off")
      else
        osc_controls[osc][CONTROL_VOLUME].tooltip = tostring(i)
        update_status("Oscillator "..tostring(osc).." volume set to "..string.format("%+2.1f",i).."dB")
      end

    elseif controlid == CONTROL_PANNING then
      local i = osc_an[osc]:get_panning()
      osc_controls[osc][CONTROL_PANNING].tooltip = tostring(i)
      update_status("Oscillator "..tostring(osc).." panning set to "..tostring(i))
    end
  end
      
end


function create_controls()

  -- Reset all controls
  osc_controls = {{}, {}, {}, {}}
  
  -- Oscillator selection
  for o = 1, 4 do
    -- [1] = osc_type
    osc_controls[o][CONTROL_OSC_TYPE] =
      vb:chooser{
        items = {"Analog", "Wavetable"},
        value = osc_type[o],
        midi_mapping = "ReSynth:Oscillator "..o..":Oscillator Type",
        notifier = function(v)
          set_oscillator_type(o, v)
          update_controls(o, CONTROL_OSC_TYPE)
        end
      }
      
      
    -- [2] = analog.waveform
    osc_controls[o][CONTROL_ANALOG_WAVEFORM] = 
      vb:chooser{
        items = {"Sine/Tri", "Pulse", "Sawtooth", "Noise"},
        value = osc_an[o]:get_waveform(),
        midi_mapping = "ReSynth:Oscillator "..o..":Analog Waveform",
        notifier = function(v)
         set_waveform(o, v)
         update_controls(o, CONTROL_ANALOG_WAVEFORM)
        end
      }
      
    -- [3] = analog.wavemod
    osc_controls[o][CONTROL_ANALOG_WAVEMOD] =
      vb:rotary{
        min = 0,
        max = 63.1,
        value = osc_an[o]:get_wavemod(),
        midi_mapping = "ReSynth:Oscillator "..o..":Analog Wavemod",
        tooltip = tostring(osc_an[o]:get_wavemod()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_ANALOG_WAVEMOD].value then
            set_wavemod(o, math.floor(v))
            update_controls(o, CONTROL_ANALOG_WAVEMOD)
          end
        end
      }
      
    -- [4] = analog.lfo depth
    osc_controls[o][CONTROL_ANALOG_LFO_DEPTH] =
      vb:rotary{
        min = 0,
        max = 31.1,
        value = osc_an[o]:get_wavemod_lfo_depth(),
        midi_mapping = "ReSynth:Oscillator "..o..":Analog Wavemod LFO depth",
        tooltip = tostring(osc_an[o]:get_wavemod_lfo_depth()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_ANALOG_LFO_DEPTH].value then
            set_wavemod_lfo_depth(o, math.floor(v))
            update_controls(o, CONTROL_ANALOG_LFO_DEPTH)
          end
        end
      }
      
    -- [5] = analog.lfo rate
    osc_controls[o][CONTROL_ANALOG_LFO_RATE] =
      vb:rotary{
        min = 0,
        max = 256.1,
        value = osc_an[o]:get_wavemod_lfo_rate(),
        midi_mapping = "ReSynth:Oscillator "..o..":Analog Wavemod LFO rate",
        tooltip = tostring(osc_an[o]:get_wavemod_lfo_rate()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_ANALOG_LFO_RATE].value then
            set_wavemod_lfo_rate(o, math.floor(v))
            update_controls(o, CONTROL_ANALOG_LFO_RATE)
          end
        end
      }
      
    -- [6] = wavetable.wavebank
    osc_controls[o][CONTROL_WAVETABLE_WAVEBANK] =
      vb:popup{
        items = osc_wt[o]:get_bank_list(),
        value = osc_wt[o]:get_wavebank(),
        notifier = function(v)
          local bank_id = osc_wt[o]:get_bank_list()[v]
          osc_controls[o][CONTROL_WAVETABLE_WAVEFORM].items =
            osc_wt[o]:get_waveform_list(bank_id)
          update_controls(o, CONTROL_WAVETABLE_WAVEBANK)
        end
      }
      
    -- [7] = wavetable.waveform
    osc_controls[o][CONTROL_WAVETABLE_WAVEFORM] =
      vb:popup{
        items = {" "},
        value = 1,
        notifier = function(v)
          local wbank = osc_wt[o]:get_bank_list()[osc_controls[o][CONTROL_WAVETABLE_WAVEBANK].value]
          local wid = osc_wt[o]:get_waveform_list(wbank)
          set_oscillator_type(o, WAVETABLE_OSC)
          osc_wt[o]:load_waveform(wbank, wid[v])
          inst.samples[o].name = osc_wt[o]:get_settings()
          show_waveform(o)
          osc_controls[o][CONTROL_OSC_TYPE].value = WAVETABLE_OSC --auto switch to wavetalbe (2.2 update)
          update_controls(o, CONTROL_OSC_TYPE)
          update_controls(o, CONTROL_WAVETABLE_WAVEFORM)
        end
      }
      
    -- [8] = common.smoothing
    osc_controls[o][CONTROL_SMOOTHING] = 
      vb:switch{
        items = {"Calm", "Harsh"},
        value = 1,
        midi_mapping = "ReSynth:Oscillator "..o..":Smoothing",
        notifier = function(v)
          if v == 1 then  -- TODO: hacky, we just set all oscs to the same
            set_smoothing(o, true)
          else
            set_smoothing(o, false)
          end
          update_controls(o, CONTROL_SMOOTHING)
        end
      }
    
    -- [9] = common.transpose
    osc_controls[o][CONTROL_TRANSPOSE] = 
      vb:rotary{
        min = -24,
        max = 24.1,
        value = osc_an[o]:get_transpose(),  -- TODO: hacky, we just set all oscs to the same
        midi_mapping = "ReSynth:Oscillator "..o..":Transpose",
        tooltip = tostring(osc_an[o]:get_transpose()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_TRANSPOSE].value then
            set_transpose(o, math.floor(v))
            update_controls(o, CONTROL_TRANSPOSE)
          end
        end
      }

    -- [10] = common.fine_tune
    osc_controls[o][CONTROL_FINE_TUNE] = 
      vb:rotary{
        min = -63,
        max = 63.1,
        value = osc_an[o]:get_fine_tune(),  -- TODO: hacky, we just set all oscs to the same
        midi_mapping = "ReSynth:Oscillator "..o..":Fine tune",
        tooltip = tostring(osc_an[o]:get_fine_tune()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_FINE_TUNE].value then
            set_fine_tune(o, math.floor(v))
            update_controls(o, CONTROL_FINE_TUNE)
          end
        end
      }
      
    -- [11] = common.volume
    osc_controls[o][CONTROL_VOLUME] = 
      vb:rotary{
        min = -38,
        max = 3,
        value = osc_an[o]:get_volume(),  -- TODO: hacky, we just set all oscs to the same
        midi_mapping = "ReSynth:Oscillator "..o..":Volume",
        tooltip = tostring(osc_an[o]:get_volume()),
        notifier = function(v)
          if v < VOL_CUTOUT then
            -- round down to -INFDB
            set_volume(o, math.infdb)
          else
            set_volume(o,v)
          end
          update_controls(o, CONTROL_VOLUME)
        end
      }

    -- [12] = common.panning
    osc_controls[o][CONTROL_PANNING] = 
      vb:rotary{
        min = -50,
        max = 50.1,
        value = osc_an[o]:get_panning(),  -- TODO: hacky, we just set all oscs to the same
        midi_mapping = "ReSynth:Oscillator "..o..":Panning",
        tooltip = tostring(osc_an[o]:get_panning()),
        notifier = function(v)
          if math.floor(v) ~= osc_controls[o][CONTROL_PANNING].value then
            set_panning(o, math.floor(v))
            update_controls(o, CONTROL_PANNING)
          end
        end
      }
  end

  -- Create status line
  status_line = vb:text{width = 330, --580
                        text = STATUS_LINE_STARTUP,
                        font = "normal",
                       }


  -- Main dialog
  resynth_view =
  
  -- main rack
  vb:column{
    margin = DEFAULT_DIALOG_MARGIN,
    spacing = 8,
    uniform = true,
    
    -- Osc 1
    vb:row{
      margin = DEFAULT_DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      style = MAIN_GROUP_STYLE,
      uniform = true,
      -- Oscillator type group
      vb:column{
        uniform = true,
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = 10,
        style = CONTROL_GROUP_STYLE, 
        osc_controls[1][1],
        osc_controls[1][8],
      },
        
  
      -- Osc 1 Analog Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,         
        -- waveform        
        osc_controls[1][2],         
        -- wavemod
        vb:column{
          uniform = true,
          osc_controls[1][3],
          vb:text{
            text = " Mod",
          },
        },
        -- lfo depth
        vb:column{
          uniform = true,
          osc_controls[1][4],
          vb:text{
            text = "  LFO",
          },
        },
        -- lfo rate
        vb:column{
          uniform = true,
          osc_controls[1][5],
          vb:text{
            text = " Rate",
          },
        },
      },
      -- Osc 1 Wavetable Settings
      vb:column{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- wavebank
        vb:row{
          vb:text{
            text = "Wavebank:",
          },
          osc_controls[1][6],
        },
        -- waveform
        vb:row{
          vb:text{
            text = "Waveform:",
          },
          osc_controls[1][7],
        },
      },
      -- Osc 1 Tuning Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- tranpose
        vb:column{
          uniform = true,
          osc_controls[1][9],
          vb:text{
            text = " Trans",
          },
        },
        -- fine tune
        vb:column{
          uniform = true,
          osc_controls[1][10],
          vb:text{
            text = "  Fine",
          },
        },
      },
      -- Osc 1 Mixer Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- volume
        vb:column{
          uniform = true,
          osc_controls[1][11],
          vb:text{
            text = "  Vol",
          },
        },
        -- panning
        vb:column{
          uniform = true,
          osc_controls[1][12],
          vb:text{
            text = "  Pan",
          },
        },
      },
    },


    -- Osc 2
    vb:row{
      margin = DEFAULT_DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      style = MAIN_GROUP_STYLE,
      uniform = true,
      -- Oscillator type group
      vb:column{
        uniform = true,
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = 10,
        style = CONTROL_GROUP_STYLE,
        osc_controls[2][1],
        osc_controls[2][8],
      },
      -- Osc 2 Analog Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,         
        -- waveform        
        osc_controls[2][2],         
        -- wavemod
        vb:column{
          uniform = true,
          osc_controls[2][3],
          vb:text{
            text = " Mod",
          },
        },
        -- lfo depth
        vb:column{
          uniform = true,
          osc_controls[2][4],
          vb:text{
            text = "  LFO",
          },
        },
        -- lfo rate
        vb:column{
          uniform = true,
          osc_controls[2][5],
          vb:text{
            text = " Rate",
          },
        },
      },
      -- Osc 2 Wavetable Settings
      vb:column{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- wavebank
        vb:row{
          vb:text{
            text = "Wavebank:",
          },
          osc_controls[2][6],
        },
        -- waveform
        vb:row{
          vb:text{
            text = "Waveform:",
          },
          osc_controls[2][7],
        },
      },
      -- Osc 2 Tuning Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- tranpose
        vb:column{
          uniform = true,
          osc_controls[2][9],
          vb:text{
            text = " Trans",
          },
        },
        -- fine tune
        vb:column{
          uniform = true,
          osc_controls[2][10],
          vb:text{
            text = "  Fine",
          },
        },
      },
      -- Osc 2 Mixer Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- volume
        vb:column{
          uniform = true,
          osc_controls[2][11],
          vb:text{
            text = "  Vol",
          },
        },
        -- panning
        vb:column{
          uniform = true,
          osc_controls[2][12],
          vb:text{
            text = "  Pan",
          },
        },
      },
    },

    -- Osc 3
    vb:row{
      margin = DEFAULT_DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      style = MAIN_GROUP_STYLE,
      uniform = true,
      -- Oscillator type group
      vb:column{
        uniform = true,
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = 10,
        style = CONTROL_GROUP_STYLE,
        osc_controls[3][1],
        osc_controls[3][8],
      },
      -- Osc 3 Analog Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,         
        -- waveform        
        osc_controls[3][2],         
        -- wavemod
        vb:column{
          uniform = true,
          osc_controls[3][3],
          vb:text{
            text = " Mod",
          },
        },
        -- lfo depth
        vb:column{
          uniform = true,
          osc_controls[3][4],
          vb:text{
            text = "  LFO",
          },
        },
        -- lfo rate
        vb:column{
          uniform = true,
          osc_controls[3][5],
          vb:text{
            text = " Rate",
          },
        },
      },
      -- Osc 3 Wavetable Settings
      vb:column{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- wavebank
        vb:row{
          vb:text{
            text = "Wavebank:",
          },
          osc_controls[3][6],
        },
        -- waveform
        vb:row{
          vb:text{
            text = "Waveform:",
          },
          osc_controls[3][7],
        },
      },
      -- Osc 3 Tuning Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- tranpose
        vb:column{
          uniform = true,
          osc_controls[3][9],
          vb:text{
            text = " Trans",
          },
        },
        -- fine tune
        vb:column{
          uniform = true,
          osc_controls[3][10],
          vb:text{
            text = "  Fine",
          },
        },
      },
      -- Osc 3 Mixer Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- volume
        vb:column{
          uniform = true,
          osc_controls[3][11],
          vb:text{
            text = "  Vol",
          },
        },
        -- panning
        vb:column{
          uniform = true,
          osc_controls[3][12],
          vb:text{
            text = "  Pan",
          },
        },
      },
    },


    -- Osc 4
    vb:row{
      margin = DEFAULT_DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      style = MAIN_GROUP_STYLE,
      uniform = true,
      -- Oscillator type group
      vb:column{
        uniform = true,
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = 10,
        style = CONTROL_GROUP_STYLE,
        osc_controls[4][1],
        osc_controls[4][8],
      },
      -- Osc 4 Analog Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,         
        -- waveform        
        osc_controls[4][2],         
        -- wavemod
        vb:column{
          uniform = true,
          osc_controls[4][3],
          vb:text{
            text = " Mod",
          },
        },
        -- lfo depth
        vb:column{
          uniform = true,
          osc_controls[4][4],
          vb:text{
            text = "  LFO",
          },
        },
        -- lfo rate
        vb:column{
          uniform = true,
          osc_controls[4][5],
          vb:text{
            text = " Rate",
          },
        },
      },
      -- Osc 4 Wavetable Settings
      vb:column{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- wavebank
        vb:row{
          vb:text{
            text = "Wavebank:",
          },
          osc_controls[4][6],
        },
        -- waveform
        vb:row{
          vb:text{
            text = "Waveform:",
          },
          osc_controls[4][7],
        },
      },
      -- Osc 4 Tuning Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- tranpose
        vb:column{
          uniform = true,
          osc_controls[4][9],
          vb:text{
            text = " Trans",
          },
        },
        -- fine tune
        vb:column{
          uniform = true,
          osc_controls[4][10],
          vb:text{
            text = "  Fine",
          },
        },
      },
      -- Osc 4 Mixer Settings
      vb:row{
        margin = DEFAULT_DIALOG_MARGIN,
        spacing = CONTENT_SPACING,
        style = CONTROL_GROUP_STYLE,
        -- volume
        vb:column{
          uniform = true,
          osc_controls[4][11],
          vb:text{
            text = "  Vol",
          },
        },
        -- panning
        vb:column{
          uniform = true,
          osc_controls[4][12],
          vb:text{
            text = "  Pan",
          },
        },
      },
    },


    -- Status footer
    vb:row{
      margin = DEFAULT_DIALOG_MARGIN,
      spacing = CONTENT_SPACING,
      style = MAIN_GROUP_STYLE,
      uniform = true,

      -- status bar
      status_line,
      
      -- randomize osc
      vb:button{
        text = "Randomize Oscillators",
        notifier = function()
          randomize_osc()
        end
      },
      
      -- randomize tuning/pan
      vb:button{
        text = "Randomize Tuning/Mixer",
        notifier = function()
          randomize_tuningmixer()
        end
      },
      
      -- reset
      vb:button{
        text = "Reset",
        notifier = function()
          global_reset()
        end
      },
    },
    
  }
end


function keyhander(dialog, key)
  -- Keyhander for ReSynth dialog - pass back to main Renoise window
  return key
end

function display()
  if resynth_dialog then
    if resynth_dialog.visible then
      return -- already visible
    end
  end
  create_controls()
  
  -- show waveform list
  for o = 1, 4 do
    local bank_id = osc_wt[o]:get_bank_list()[osc_controls[o][CONTROL_WAVETABLE_WAVEBANK].value]
    if bank_id ~= nil then
      osc_controls[o][CONTROL_WAVETABLE_WAVEFORM].items =
        osc_wt[o]:get_waveform_list(bank_id)
      update_controls(o, CONTROL_WAVETABLE_WAVEBANK)
    end
  end
  
  resynth_dialog = renoise.app():show_custom_dialog("ReSynth2",
                                                     resynth_view,
                                                     keyhander)
end
