--------------------------------------------------------------------------------
-- ReSynth Tool
--
-- Copyright 2011 Martin Bealby
--
-- Midi map handlers
--
-- TODO: Rationalise common control updates with gui.lua (.value & .tooltip)
--------------------------------------------------------------------------------


--------------------------------------------------------------------------------
-- Tool initialisation
--------------------------------------------------------------------------------
for o = 1, 4 do
  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Oscillator Type",
    invoke = function(message)
      if message.int_value > 63 then
        -- wavetable
        set_oscillator_type(o, WAVETABLE_OSC)
        osc_controls[o][CONTROL_OSC_TYPE].value = WAVETABLE_OSC
      else
        -- analog
        set_oscillator_type(o, ANALOG_OSC)
        osc_controls[o][CONTROL_OSC_TYPE].value = ANALOG_OSC
      end
      update_controls(o, CONTROL_OSC_TYPE)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Analog Waveform",
    invoke = function(message)
      local val = math.floor(scale_midi_param(1,
                                              #osc_controls[o][CONTROL_ANALOG_WAVEFORM].items,
                                              message.int_value))
      set_waveform(o, val)
      osc_controls[o][CONTROL_ANALOG_WAVEFORM].value = val
      update_controls(o, CONTROL_ANALOG_WAVEFORM)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Analog Wavemod",
    invoke = function(message)
      set_wavemod(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_ANALOG_WAVEMOD].min,
                                                 osc_controls[o][CONTROL_ANALOG_WAVEMOD].max,
                                                 message.int_value)))
      osc_controls[o][CONTROL_ANALOG_WAVEMOD].value = osc_an[o]:get_wavemod()
      update_controls(o, CONTROL_ANALOG_WAVEMOD)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Analog Wavemod LFO depth",
    invoke = function(message)
      set_wavemod_lfo_depth(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_ANALOG_LFO_DEPTH].min,
                                                           osc_controls[o][CONTROL_ANALOG_LFO_DEPTH].max,
                                                           message.int_value)))
      osc_controls[o][CONTROL_ANALOG_LFO_DEPTH].value = osc_an[o]:get_wavemod_lfo_depth()
      update_controls(o, CONTROL_ANALOG_LFO_DEPTH)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Analog Wavemod LFO rate",
    invoke = function(message)
      set_wavemod_lfo_rate(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_ANALOG_LFO_RATE].min,
                                                          osc_controls[o][CONTROL_ANALOG_LFO_RATE].max,
                                                          message.int_value)))
      osc_controls[o][CONTROL_ANALOG_LFO_RATE].value = osc_an[o]:get_wavemod_lfo_rate()
      update_controls(o, CONTROL_ANALOG_LFO_RATE)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Smoothing",
    invoke = function(message)
      if message.int_value > 63 then
        -- harsh
        set_smoothing(o, false)
        osc_controls[o][CONTROL_SMOOTHING].value = 2
      else
        -- calm
        set_smoothing(o, true)
        osc_controls[o][CONTROL_SMOOTHING].value = 1
      end
      update_controls(o, CONTROL_SMOOTHING)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Transpose",
    invoke = function(message)
      set_transpose(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_TRANSPOSE].min,
                                                   osc_controls[o][CONTROL_TRANSPOSE].max,
                                                   message.int_value)))
      osc_controls[o][CONTROL_TRANSPOSE].value = osc_an[o]:get_transpose()
      update_controls(o, CONTROL_TRANSPOSE)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Fine tune",
    invoke = function(message)
      set_fine_tune(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_FINE_TUNE].min,
                                                   osc_controls[o][CONTROL_FINE_TUNE].max,
                                                   message.int_value)))
      osc_controls[o][CONTROL_FINE_TUNE].value = osc_an[o]:get_fine_tune()
      update_controls(o, CONTROL_FINE_TUNE)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Volume",
    invoke = function(message)
      set_volume(o, scale_midi_param(osc_controls[o][CONTROL_VOLUME].min,
                                     osc_controls[o][CONTROL_VOLUME].max,
                                     message.int_value))
      osc_controls[o][CONTROL_VOLUME].value = osc_an[o]:get_volume()
      update_controls(o, CONTROL_VOLUME)
    end
    }

  renoise.tool():add_midi_mapping{
    name = "ReSynth:Oscillator "..o..":Panning",
    invoke = function(message)
      set_panning(o, math.floor(scale_midi_param(osc_controls[o][CONTROL_PANNING].min,
                                                 osc_controls[o][CONTROL_PANNING].max,
                                                 message.int_value)))
      osc_controls[o][CONTROL_PANNING].value = osc_an[o]:get_panning()
      update_controls(o, CONTROL_PANNING)
    end
    }
end
